#include <m8c.h>        // part specific constants and macros
#include <stdio.h>
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include "main.h"
#include "eeprom.h"


controllerConfig_t currentConfig;
userData_t tempUser;

extern char TX_Data[64];

unsigned char buf[EEPROM_BLOCK_SIZE];
unsigned char buf2[8];
unsigned char *buf3;

unsigned short 	toCheckFacilityCode;
unsigned long 	toCheckUserCode;
unsigned char	toCheckDoorNum;

// read configuration data from EEPROM
void eeprom_load_config(void)
{
	
	E2PROM_E2Read(CONFIG_START_ADDRESS, buf, EEPROM_BLOCK_SIZE);
	currentConfig = *((controllerConfig_t*)buf);
	
	
	if (currentConfig.isFlashConfigured == 0x30)	// detect if the PSoC was just reprogrammed
	{
		dprintf("Flash not configured.  Setting initial parameters.\r\n");
		eeprom_erase();
		currentConfig.isFlashConfigured = 0;
		currentConfig.numUsers = 1;
		currentConfig.useExternalAuth = 0;
		eeprom_write_config();
		eeprom_test();  
	} 
	else
	{
		dprintf("Flash is configured.\r\n");
		eprintf("Num Users: %d\r\nR1: %x\r\n", currentConfig.numUsers, currentConfig.readerConfig[0]);
	}
}

// erase all 4906 bytes of eeprom flash
void eeprom_erase(void)
{
	int i;
	for(i=0;i<EEPROM_BLOCK_SIZE;i++)
		buf[i] = 0;
		
	for(i=0;i<64;i++)
		E2PROM_bE2Write(i*EEPROM_BLOCK_SIZE, buf, EEPROM_BLOCK_SIZE, EEPROM_TEMPERATURE);

	
}

// write the current configuration to flash
void eeprom_write_config(void)
{
	int bError;
	
	buf3 = (char *)&currentConfig;
	
    bError = E2PROM_bE2Write(CONFIG_START_ADDRESS, buf3, EEPROM_BLOCK_SIZE, EEPROM_TEMPERATURE);
     
   	if ( bError == E2PROM_NOERROR )
    {
		dprintf("EEPROM write success!\r\n");
    } else {
		eprintf("EEPROM write error: %i\r\n", bError);
	}
}

// read a block of 64 bytes from flash
int eeprom_read_block(unsigned char* block_data, unsigned char block)
{
	int bError;
	// data stored in 64 blocks of 64 bytes 
	// block 0 is config. every other block is a user block. each user block has 8 users
	// block range is 0 to 63 
	
	if (block > 63)
	{
		return -3;			// error out
	}

	E2PROM_E2Read(EEPROM_BLOCK_SIZE * block, block_data, EEPROM_BLOCK_SIZE);
	
	return 0;
}

// write a block of 64 bytes to flash
int eeprom_write_block(unsigned char* block_data, unsigned char block)
{
	int bError;
	
	// data stored in 64 blocks of 64 bytes 
	// block 0 is config. every other block is a user block. each user block has 8 users
	// block range is 0 to 63 
	
	if (block > 63)
	{
		return -3;			// error out
	}

	bError = E2PROM_bE2Write(EEPROM_BLOCK_SIZE * block, block_data, EEPROM_BLOCK_SIZE, EEPROM_TEMPERATURE);
	
	if (block == 0 && bError == E2PROM_NOERROR)	// block 0 is the config block, so we need load this into memory
	{
		currentConfig = *((controllerConfig_t*)block_data);
	}
	
	return bError;
}

// searches flash for a user that matches the usercode and facility code, and checks for access to a particular door
int eeprom_check_user(unsigned short facilityCode, unsigned long userCode, unsigned char doorNum)
{
	// flash is stored in blocks of 64 bytes, but reading flash takes time
	// so we break this down by reading flash a block at a time, then check all eight users in that block.
	unsigned short i, block, max_block=0;
	
	if (currentConfig.numUsers > 0)
		max_block = (currentConfig.numUsers-1) / 8;		// with 8 users per block, calculate the largest block we need to search

	// known issue: we scan all 8 users per block, even if we don't need to (for example, 10 users, but we scan all 16 spots).  
	// the door programmer software should send all 0's for these spots, so it shouldn't be a problem.
	
	for (block=0; block<=max_block; block++)			// cycle through all blocks
	{
		E2PROM_E2Read(USERS_START_ADDRESS+(EEPROM_BLOCK_SIZE * block), buf, EEPROM_BLOCK_SIZE);	// read the block from flash
		
		for (i=0; i<=(EEPROM_BLOCK_SIZE-USER_DATA_SIZE); i+=USER_DATA_SIZE)						// cycle through all users in the block
		{
			tempUser = *((userData_t*)(buf+i));			// format the raw data as a userData structure
			if (facilityCode == tempUser.facilityCode && userCode == tempUser.userCode)	// check if we have a user match
			{
				if (ATTRIB_DOOR(tempUser.attrib, doorNum))	// check if they have permissions on this door
				{
					return 1;
				} 
			}
			
		}
	}
	
	return 0;
	
}

void eeprom_test(void)
{
	int bError, i;
		
	//currentConfig.numUsers = 5;
	//currentConfig.readerConfig[0] = 0xA1;	
	//eeprom_write_config();
	
	tempUser.facilityCode = 1097;
	tempUser.userCode = 10245;
	tempUser.attrib = 0x0F;
	
	
	buf3 = (char *)&tempUser;
	
    bError = E2PROM_bE2Write(USERS_START_ADDRESS, buf3, 64, EEPROM_TEMPERATURE);
  
}

/*
	
	//unsigned char 	buf2[64];
	
	
	userData_t users[8];
	userData_t *printUser;
	unsigned char *buf3;
	
	users[0].facilityCode = 1097;
	users[0].userCode = 10245;
	
	users[1].facilityCode = 1096;
	users[1].userCode = 360883;
	
	users[2].facilityCode = 1598;
	users[2].userCode = 8508;

	buf3 = (char *)&users;

    bError = E2PROM_bE2Write(0, buf3, 64, 25);
     
   	if ( bError == E2PROM_NOERROR )
    {
		E2PROM_E2Read(0, buf1, 64 );
		csprintf(TX_Data, "EEPROM: ");
		UART_PutString(TX_Data);
		for (i=0;i<64;i++)
		{
			csprintf(TX_Data, "%x ", buf1[i]);
			UART_PutString(TX_Data);
		}
		UART_CPutString("\r\n");
		
		//printUser = (userData_t*)buf1;
		
		//printUser = &buf1[8];
		
		//csprintf(TX_Data, "User 0: %lu, %lu\r\n", buf1[0], buf1[3]);
		csprintf(TX_Data, "User 0: %lu, %lu\r\n", printUser->facilityCode, printUser->userCode);
		UART_PutString(TX_Data);
		
    } else {
		csprintf(TX_Data, "EEPROM Error: %i\r\n", bError);
		UART_PutString(TX_Data);
	}
*/



