#ifndef EEPROM_H_
#define EEPROM_H_

// EEPROM flash start address 10is at 0x7000 (block 448) and is 4096 bytes long
// the addresses used here are relative to the EEPROM flash start address, they are NOT absolute

#define CONFIG_START_ADDRESS	0
#define USERS_START_ADDRESS		64
#define EEPROM_TEMPERATURE 		25
#define EEPROM_BLOCK_SIZE		64

#define USER_DATA_SIZE		8

#define ATTRIB_DOOR(var, door) (var & (1 << (door-1)))

struct userData		// total size is 8 bytes
{
    unsigned short facilityCode; 	// 2 bytes
    unsigned long userCode;			// 4 bytes
	unsigned char attrib;			// bitfield
	// attrib[0] = permission on R1
	// attrib[1] = permission on R2
	// attrib[2] = permission on R3
	// attrib[3] = permission on R4
	unsigned char temp;
} ;

typedef struct userData userData_t;

struct controllerConfig	// total size is 64 bytes
{
	unsigned char isFlashConfigured;	// 1 byte
	unsigned short numUsers;			// 2 bytes
	unsigned char readerConfig[4];		// 4 bytes	- if 1, the reader is enabled.  if 0, the port is in GPIO mode
	unsigned char useExternalAuth;		// 1 byte   - if 1, the controller will only open doors when given the open command over serial
	
	unsigned char temp[56];
};
typedef struct controllerConfig controllerConfig_t;



void eeprom_erase(void);
void eeprom_test(void);
void eeprom_load_config(void);
void eeprom_write_config(void);
int eeprom_write_block(unsigned char* block_data, unsigned char block);
int eeprom_read_block(unsigned char* block_data, unsigned char block);
int eeprom_check_user(unsigned short facilityCode, unsigned long userCode, unsigned char doorNum);

#endif  // EEPROM_H_