//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include <stdio.h>
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

#include "main.h"
#include "eeprom.h"

// IMPORTANT: compiler must be set up to use unsigned longs with printf!
// http://www.cypress.com/?id=4&rID=43630
// basically, add this line to local.mk:
// CODECOMPRESSOR:=$(CODECOMPRESSOR) -llpm8c



#define MAX_BITS 100
#define MIN_BITS 10				// minmum number of bits for a read to be valid
#define WEIGAND_WAIT_TIME	250	// timeout before we stop looking for new weigand bits
#define RX_BUFFER_SIZE 78

#define INCREMENT_BIT bitCount++; flagDone = 0; weigand_counter = WEIGAND_WAIT_TIME;  

unsigned char databits[MAX_BITS];
unsigned char bitCount;
unsigned char flagDone;
unsigned int weigand_counter;

unsigned long lngFacCode;
unsigned long lngUserCode;


unsigned int  R1_actionCounter=0; 
unsigned char R1_actionType=0;
unsigned int  R2_actionCounter=0; 
unsigned char R2_actionType=0;
unsigned int  R3_actionCounter=0; 
unsigned char R3_actionType=0;
unsigned int  R4_actionCounter=0; 
unsigned char R4_actionType=0;

unsigned int actionCounter;
unsigned char count;

unsigned int activeReader=0;


char LCD_Data[];
char TX_Data[64];


unsigned int toggle=1;
BYTE numReads=0;

extern controllerConfig_t currentConfig;

BYTE rcv_buffer[RX_BUFFER_SIZE];
volatile unsigned short rcv_buffer_size=0;
volatile unsigned char rcv_flag=0;
unsigned char rx_timeout_counter=0;

unsigned char dataBuffer[64];

unsigned char expectedBufferSize=0;
unsigned char bSerialBinaryMode=0;

void main(void)
{
	BYTE bError;
	short i;
	
	Initial();
	dprintf("\r\n\r\n ****************** \r\n\r\n" );
	
	
	eeprom_load_config();
	
	doorAction(0,0);
	setActiveReader(0);
	
	toggle = 1;
	
	dprintf(  "\n\rM0:Door Controller v2.2\r\n" );
	dprintf(  "M0:Designed by Daniel Smith\r\n" );
	dprintf(  "M0:for GT-IEEE and WREK \r\n");
	 
	while (1)
	{

		//M8C_ClearWDT;
		if (! ++count) {	// happens every .8 ms
			
			//PIN2_6_TOGGLE;
       			
			if(rcv_flag) 	// Check if a serial byte received
			{
				rcv_flag = 0;
				if (bSerialBinaryMode) 	// SERIAL BINARY MODE (default is ASCII)
				{
					// the binary serial protocol is very simple: the first byte is the length of the serial message
					if (expectedBufferSize == 0) {							// if we haven't received anything
						expectedBufferSize = rcv_buffer[0];			// set the expected buffer size to be the first byte
						rcv_buffer_size = 0;
						rx_timeout_counter = 1;
					} else {
						rx_timeout_counter = 1;
						//rcv_buffer[rcv_buffer_size++] = UART_bReadRxData();	// add data to buffer
						
						if (rcv_buffer_size == expectedBufferSize)			// did we get all the bytes?  now we can start processing data
						{
							switch (rcv_buffer[0])
							{
								case 0x01:	// ping
									UART_PutChar(0x01);
									UART_PutChar(0x01);
									break;
									
								case 0x10:	// read EEPROM
									if (rcv_buffer_size != 2)	// command[0], user row[1]
									{
										UART_PutChar(0x01);
										UART_PutChar(0xFE);	// error
									} else {
										eeprom_read_block(dataBuffer, rcv_buffer[1]);
										UART_PutChar(EEPROM_BLOCK_SIZE + 2);	// send data size
										UART_PutChar(0x10);						// command
										UART_PutChar(rcv_buffer[1]);			// user row
										
										// send data bytes
										for (i=0;i<EEPROM_BLOCK_SIZE;i++)
										{
											UART_PutChar(dataBuffer[i]);	// send data
										}
									}
									break;
									
								case 0x11:	// write EEPROM user block
									if (rcv_buffer_size != EEPROM_BLOCK_SIZE + 2)	// command[0], user row[1], data[2-65]
									{
										UART_PutChar(0x01);
										UART_PutChar(0xFE);	// error
									} else {
										if(eeprom_write_block(&rcv_buffer[2], rcv_buffer[1]) == 0)
										{
											UART_PutChar(0x01);
											UART_PutChar(0x02);	// eeprom write success
										} else {
											UART_PutChar(0x01);
											UART_PutChar(0xFD);	// eeprom write error
										}
									}
									break;
									
								case 0x20:	// set ascii mode
									bSerialBinaryMode = 0;
									dprintf("M0: Now in serial ASCII mode.\r\n");
									break;
									
								case 0x30:	// lock all readers
									setActiveReader(5);
									UART_PutChar(0x01);
									UART_PutChar(0x30);
									break;
									
								case 0x31:	// unlock all readers
									setActiveReader(0);
									UART_PutChar(0x01);
									UART_PutChar(0x31);								
									break;
									
								default:
									UART_PutChar(0x01);
									UART_PutChar((unsigned char)0xFF);	// send a "wtf?"
								
							}
							rcv_buffer_size = 0;
							rx_timeout_counter = 0;
							expectedBufferSize = 0;
						}
					}
					
				
				} 
				else				  // SERIAL ASCII MODE
				{
					//rcv_buffer[rcv_buffer_size++] = UART_bReadRxData();
					//UART_PutString(&rcv_buffer[rcv_buffer_size-1]);	// this line provides console echo
					if(rcv_buffer[rcv_buffer_size-1] == 13 || rcv_buffer_size >= RX_BUFFER_SIZE) 
					{  
						if (rcv_buffer_size < 4)
						{
							dprintf("M0: Command too short.  Use format XN:Y\r\n");
						} 
						else 
						{
							// command structure: XN:Y
							//	X = command		rcv_buffer[0]
							// 	N = address		rcv_buffer[1]
							//  Y = data		rcv_buffer[3]
							switch (rcv_buffer[0]) 
							{
								case 'D':	// DOOR CONTROL
									rcv_buffer[1] = rcv_buffer[1] - '0'; // convert ASCII number into real number
									if (rcv_buffer[3] == 'A')
										doorAction(rcv_buffer[1], 1);
									else if (rcv_buffer[3] == 'D')
										doorAction(rcv_buffer[1], 2);
									break;
								case 'M':	// MESSAGE
									if (rcv_buffer[3] == 'P') {	// PING
										UART_CPutString("M0:P\r\n");				// respond with a pong
									} else {
										UART_CPutString("M0:?\r\n");
									}
									break;
								case 'L':	// lock or unloack readers
									if (rcv_buffer[3] == '1') {	// PING
										setActiveReader(5);			
									} else {
										setActiveReader(0);
									}
									break;
								case 'A':	// ABOUT
									if (rcv_buffer[3] = 'D')
										UART_CPutString("M0: Door Controller v2.2 for GT-IEEE HWT and WREK, written by Daniel Smith (Fall 2012)\r\n");
									break;
								case 'E':
									//UART_CPutString("EEPROM command received.\r\n");
									//currentConfig.numUsers = rcv_buffer[3]-'0';
									//eeprom_write_config();
										eeprom_read_block(dataBuffer, 1);
										dprintf("Flash user row 1: ");
										
										// send data bytes
										for (i=0;i<EEPROM_BLOCK_SIZE;i++)
										{
											eprintf("%0X ", dataBuffer[i]);	// send data
										}
										dprintf("\r\n");
									break;
								case 'B':	// enable binary mode	
									bSerialBinaryMode = 1;
									expectedBufferSize = 0;
									UART_PutChar(0x01);
									UART_PutChar(0x21);	// tell computer we acknowledge
									break;
								default:
									eprintf("M0: Unrecognized command. (%c [0x%02X], length=%i)\r\n",rcv_buffer[0],rcv_buffer[0], rcv_buffer_size);
									//strlen();
									break;
							}	// end switch
							
						} // end if (rcv_buffer_size < 4)
						rcv_buffer_size = 0;
					} // end if(rcv_buffer[rcv_buffer_size-1] == 13 || rcv_buffer_size >= RX_BUFFER_SIZE) 
				}	// end if (bSerialBinaryMode)
			} 	// end if (rcv_flag)
			
			if (bSerialBinaryMode)
			{
				if (++rx_timeout_counter == 255)	// we've been in binary mode for too long. go back to ASCII
				{
					rcv_buffer_size = 0;
					rx_timeout_counter = 0;
					expectedBufferSize = 0;
					bSerialBinaryMode = 0;
					if (activeReader == 5)	// were the readers locked? unloack them
						setActiveReader(0);
				}
			}
			
			if (++actionCounter == 94) { // happens every ~50 ms  (.8ms*63)
				
				actionCounter = 0;
				//STATUS_LED_TOGGLE;
				
		
				// BEEPER CONTROLLER ---------------------------------------------------------
				
				// this is done with concatination macros because I am lazy and this is the easiest way to do it (I think)
				// see main.h for definitions
				
				READER_CONTROL_LOOP(1);
				READER_CONTROL_LOOP(2);
				READER_CONTROL_LOOP(3);
				READER_CONTROL_LOOP(4);
				
				
				// END BEEPER CONTROLLER -----------------------------------------------------
				
			}
	

	
			// WIEGAND TIMEOUT CONTROLLER 
			// This waits to make sure that there have been no more data pulses before processing data
			if (!flagDone) {
				if (--weigand_counter == 0)
				{
					flagDone = 1;	
				}
			}
	
			// check if we have recieved bits, and haven't had any more in a while.
			if (bitCount > 0 && flagDone) {
				if (bitCount >= MIN_BITS)	//check if we have enough bits for a valid read
				{
					processBits();
					
					numReads++;
					
					if (!currentConfig.useExternalAuth)	// only check EEPROM user credentials if configured.  otherwise we must receive a serial command to open a door
					{
						if (eeprom_check_user((unsigned short)lngFacCode, lngUserCode, activeReader))
						{
							doorAction(activeReader, 1);
							eprintf("D%i:A\r\n", activeReader);
						} else {
							doorAction(activeReader, 2);
							eprintf("D%i:D\r\n", activeReader);
						}
					}
					
					bitCleanup();
					setActiveReader(0);
									
				} else {
					// do nothing if not enough bits
					bitCleanup();
				}
				
			}
		}	
	}
 
}


void doorAction(unsigned char doorNum, unsigned char doorAction) {
	/* 
	Performs a door action (accept, deny)
	"accept" will do one long beep, with the LED on for several seconds.  Door strike should be tied to LED
		doorAction = 1 -> two long beep ("accept")
		doorAction = 2 -> three short beeps ("deny")
	If doorNum is 0, turns off all beepers and strikes
	*/

	switch (doorNum) {
		case 0:
			R1_BEEP_OFF;
			R2_BEEP_OFF;
			R3_BEEP_OFF;
			R4_BEEP_OFF;
			R1_STRIKE_OFF;
			R2_STRIKE_OFF;
			R3_STRIKE_OFF;
			R4_STRIKE_OFF;
			break;
		case 1:
			R1_BEEP_ON;
			//R1_LED_ON;
			if (doorAction == 1) { R1_STRIKE_ON; } else { R1_STRIKE_OFF; }
			R1_actionType = doorAction;
			R1_actionCounter = 0;
			break;
		case 2:
			R2_BEEP_ON;
			//R2_LED_ON;
			if (doorAction == 1) { R2_STRIKE_ON; } else { R2_STRIKE_OFF; }
			R2_actionType = doorAction;
			R2_actionCounter = 0;
			break;		
		case 3:
			R3_BEEP_ON;
			//R3_LED_ON;
			if (doorAction == 1) { R3_STRIKE_ON; } else { R3_STRIKE_OFF; }
			R3_actionType = doorAction;
			R3_actionCounter = 0;
			break;	
		case 4:
			R4_BEEP_ON;
			//R4_LED_ON;
			if (doorAction == 1) { R4_STRIKE_ON; } else { R4_STRIKE_OFF; }
			R4_actionType = doorAction;
			R4_actionCounter = 0;
			break;	
	}


}

void setActiveReader(unsigned char doorNum) {
	/*
	Holds all readers except current
	Each HID reader has a "hold" input.  When high, the reader behaves normally.
		When low, the reader will read a card, but will not send the data until
		the hold input is high again.  The reader will only read one card.  This is so
		that we don't have to handle simulatenous processing of multiple readers.

	doorNum is the reader we DON'T want to hold (the active reader!).
		Set it 0 to resume all readers to normal state.
	*/

	if (activeReader == doorNum && doorNum) {
		return;
	}

	activeReader = doorNum;

	if (doorNum == 0) {
		R1_HOLD_OFF; R2_HOLD_OFF; R3_HOLD_OFF; R4_HOLD_OFF; 
	} else {
		R1_HOLD_ON; R2_HOLD_ON; R3_HOLD_ON; R4_HOLD_ON; 
		switch (doorNum) {
			case 1:
				R1_HOLD_OFF;
				break;
			case 2:
				R2_HOLD_OFF;
				break;
			case 3:
				R3_HOLD_OFF;
				break;
			case 4:
				R4_HOLD_OFF;
				break;
			
		}
	}			
	

}

void processBits(void)
{
	// we will decode the bits differently depending on how many bits we have
    // see www.pagemac.com/azure/data_formats.php for more info
	unsigned char i;
	//UART_CPutString("\r\n");
	
	if (bSerialBinaryMode)	// are we in serial binary mode?
	{
		UART_PutChar(0x01);	// tell the computer we are going back to serial ASCII mode
		UART_PutChar(0x20);
		bSerialBinaryMode = 0;
	}
	
	switch (bitCount)
	{
		case 35:
			// 35 bit HID Corporate 1000 format
			// facility code = bits 2 to 14
			for (i=2; i<14; i++)
			{
				lngFacCode <<=1;
				lngFacCode |= databits[i];
			}
			
			// card code = bits 15 to 34
			for (i=14; i<34; i++)
			{
				lngUserCode <<=1;
				lngUserCode |= databits[i];
			}
			break;
		case 26:
			// standard 26 bit format
			// facility code = bits 2 to 9
			for (i=1; i<9; i++)
			{
				lngFacCode <<=1;
				lngFacCode |= databits[i];
			}
			
			// card code = bits 10 to 23
			for (i=9; i<25; i++)
			{
				lngUserCode <<=1;
				lngUserCode |= databits[i];
			}
			break;
		case 34:
			// standard 34 bit format
			// facility code = bits 2 to 9
			for (i=1; i<17; i++)
			{
				lngFacCode <<=1;
				lngFacCode |= databits[i];
			}
			
			// card code = bits 10 to 23
			for (i=17; i<33; i++)
			{
				lngUserCode <<=1;
				lngUserCode |= databits[i];
			}
			break;
		default:
			// unable to decode
			eprintf("M%i: Unable to decode with %i bits!\r\n", activeReader, bitCount);
			bitCleanup();
			return;
    }

/*
	clearLCD();
	LCD_Position(0,0);
	csprintf(LCD_Data, "%d", lngFacCode);
	LCD_PrString(LCD_Data);
	
	LCD_Position(0,4);
	csprintf(LCD_Data, "%d", lngUserCode);
	LCD_PrString(LCD_Data);
	
	LCD_Position(0,13);
	csprintf(LCD_Data, "%i", bitCount);
	LCD_PrString(LCD_Data);
*/	
	
	eprintf("R%i: %i, %lu, ", activeReader, bitCount, lngUserCode);
	eprintf("%d\r\n", lngFacCode);

	
	

}

void clearLCD(void)
{
	//LCD_Position(0,0);
	//LCD_PrCString("                "); 	
	//LCD_Position(1,0);
	//LCD_PrCString("                "); 	
}

void bitCleanup(void)
{
	// cleanup and get ready for the next card
	unsigned char i;
	
	bitCount = 0;
	lngUserCode = 0;
	lngFacCode = 0;
	
	for (i=0; i<MAX_BITS; i++) 
	{
		databits[i] = 0;
	}
}


void Initial(void)
{
	int i;

	
	flagDone = 1;
	bitCount = 0;
	weigand_counter = WEIGAND_WAIT_TIME;
	
	bitCleanup();

//	LCD_Start();                    // Initialize LCD hardware/
//	LCD_Position(0,0);              // Position cursor @ row 0, col 4
//	LCD_PrCString("RFID Reader");      // Print a constant "ROM" string
	
 	M8C_EnableIntMask(INT_MSK0, INT_MSK0_GPIO);   	// enable GPIO interrupts
 	M8C_EnableGInt;									// enable global interrupts
	
    //UART_CmdReset();                      // Initialize receiver/cmd buffer
   UART_IntCntl(UART_ENABLE_RX_INT);     // Enable RX interrupts     
    UART_Start(UART_PARITY_NONE);         // Enable UART
	//UART_CPutString("Hello World!\r\n\r\n");
	
	//M8C_EnableWatchDog(); 	// enable watchdog
}


// GPIO interrupt routine
// boot.tpl must have ljmp for GPIO
#pragma interrupt_handler HiPriISR
void HiPriISR(void)
{
	if(R1_DATA1 == 0){					// Reader 1: "1" bit
		databits[bitCount] = 1;
		setActiveReader(1);
		//UART_CPutString("1");
		INCREMENT_BIT;
	}
	
	else if(R1_DATA0 == 0){					// Reader 1: "0" bit
		setActiveReader(1);
		//UART_CPutString("0");
		INCREMENT_BIT;
	}
	
	
	else if(R2_DATA1 == 0){					// Reader 2: "1" bit
		databits[bitCount] = 1;
		setActiveReader(2);
		INCREMENT_BIT;
	}
	
	else if(R2_DATA0 == 0){					// Reader 2: "0" bit
		setActiveReader(2);
		INCREMENT_BIT;
	}

	else if(R3_DATA1 == 0){					// Reader 3: "1" bit
		databits[bitCount] = 1;
		setActiveReader(3);
		INCREMENT_BIT;
	}
	
	else if(R3_DATA0 == 0){					// Reader 3: "0" bit
		setActiveReader(3);
		INCREMENT_BIT;
	}
	
	else if(R4_DATA1 == 0){					// Reader 4: "1" bit
		databits[bitCount] = 1;
		setActiveReader(4);
		INCREMENT_BIT;
	}
	
	else if(R4_DATA0 == 0){					// Reader 4: "0" bit
		setActiveReader(4);
		INCREMENT_BIT;
	}
	
}

// UART RX interrupt routine
// boot.tpl must have ljmp for UART RX block
#pragma interrupt_handler UART_ISR
void UART_ISR(void)
{
	if(UART_bReadRxStatus() & UART_RX_REG_FULL) 	// Check if serial byte received
	{
		rcv_buffer[rcv_buffer_size] = UART_bReadRxData();
		rcv_buffer_size++;
		rcv_flag = 1;
	}
	STATUS_LED_TOGGLE;
}