#ifndef MAIN_H_
#define MAIN_H_

#define BIT0	0b00000001
#define BIT1	0b00000010
#define BIT2	0b00000100
#define BIT3	0b00001000
#define BIT4	0b00010000
#define BIT5	0b00100000
#define BIT6	0b01000000
#define BIT7	0b10000000

// ALL READER CONTROL SIGNALS ARE ACTIVE LOW!!!
// ex: "BEEP_ON" means set the pin LOW

// this sets up pin configurations for the readers
#define R1_DATA0		(PRT0DR & BIT7)
#define R1_DATA1		(PRT0DR & BIT5)
#define R1_BEEP_OFF		PRT0DR |=  BIT3
#define R1_BEEP_ON		PRT0DR &= ~BIT3
#define R1_HOLD_OFF		PRT0DR |=  BIT1
#define R1_HOLD_ON		PRT0DR &= ~BIT1
#define R1_STRIKE_ON	PRT2DR |=  BIT7		// strike is active high
#define R1_STRIKE_OFF	PRT2DR &= ~BIT7

#define R2_DATA0		(PRT2DR & BIT5)
#define R2_DATA1		(PRT2DR & BIT3)
#define R2_BEEP_OFF		PRT2DR |=  BIT1
#define R2_BEEP_ON		PRT2DR &= ~BIT1
#define R2_HOLD_OFF		PRT1DR |=  BIT7
#define R2_HOLD_ON		PRT1DR &= ~BIT7
#define R2_STRIKE_ON	PRT1DR |=  BIT5		// strike is active high
#define R2_STRIKE_OFF	PRT1DR &= ~BIT5

#define R3_DATA0		(PRT1DR & BIT3)
#define R3_DATA1		(PRT1DR & BIT2)
#define R3_BEEP_OFF		PRT1DR |=  BIT4
#define R3_BEEP_ON		PRT1DR &= ~BIT4
#define R3_HOLD_OFF		PRT1DR |=  BIT6
#define R3_HOLD_ON		PRT1DR &= ~BIT6
#define R3_STRIKE_ON	PRT2DR |=  BIT0		// strike is active high
#define R3_STRIKE_OFF	PRT2DR &= ~BIT0

#define R4_DATA0		(PRT2DR & BIT2)
#define R4_DATA1		(PRT2DR & BIT4)
#define R4_BEEP_OFF		PRT2DR |=  BIT6
#define R4_BEEP_ON		PRT2DR &= ~BIT6
#define R4_HOLD_OFF		PRT0DR |=  BIT0
#define R4_HOLD_ON		PRT0DR &= ~BIT0
#define R4_STRIKE_ON	PRT0DR |=  BIT2		// strike is active high
#define R4_STRIKE_OFF	PRT0DR &= ~BIT2

#define STATUS_LED_TOGGLE	PRT1DR ^=  BIT0
#define STATUS_LED_ON		PRT1DR |=  BIT0
#define STATUS_LED_OFF		PRT1DR &= ~BIT0

// concatination macros for controlling beepers and strikes
#define STRIKE_ON(READER) R ## READER ## _STRIKE_ON
#define STRIKE_OFF(READER) R ## READER ## _STRIKE_OFF
#define BEEP_ON(READER) R ## READER ## _BEEP_ON
#define BEEP_OFF(READER) R ## READER ## _BEEP_OFF
#define ACTIONCOUNTER(READER)	(R ## READER ## _actionCounter)
#define ACTIONTYPE(READER)	(R ## READER ## _actionType)

#define DOOR_OPEN_TIME	(3 * 20)			// value in 50 ms units

// macro that operates the timing for reader beepers and strikes
/*
#define READER_CONTROL_LOOP(i)	\
					++ACTIONCOUNTER(i);	\
					if (ACTIONTYPE(i) == 1)	{ 				\
						if (ACTIONCOUNTER(i) == 1 || ACTIONCOUNTER(i) == 3) { \
							BEEP_OFF(i); \
						} else if (ACTIONCOUNTER(i) == 2) { \
							BEEP_ON(i); \
						} else if (ACTIONCOUNTER(i) >= DOOR_OPEN_TIME) { \
							ACTIONTYPE(i) = 0; \
							STRIKE_OFF(i); \
						} \
					} else if (ACTIONTYPE(i) == 2) {		\
						if (ACTIONCOUNTER(i) == 1 || ACTIONCOUNTER(i) == 3 || ACTIONCOUNTER(i) == 5) { \
							BEEP_OFF(i); \
						} else if (ACTIONCOUNTER(i) == 2 || ACTIONCOUNTER(i) == 4) { \
							BEEP_ON(i); \
						} else if (ACTIONCOUNTER(i) >= 5) { \
							ACTIONTYPE(i) = 0; \
						} \
					} \
*/

#define READER_CONTROL_LOOP(i)	\
					++ACTIONCOUNTER(i);	\
					if (ACTIONTYPE(i) == 1)	{ 				\
						if (ACTIONCOUNTER(i) == 10) { \
							BEEP_OFF(i); \
						} else if (ACTIONCOUNTER(i) >= DOOR_OPEN_TIME) { \
							ACTIONTYPE(i) = 0; \
							STRIKE_OFF(i); \
						} \
					} else if (ACTIONTYPE(i) == 2) {		\
						if (ACTIONCOUNTER(i) == 1 || ACTIONCOUNTER(i) == 3 || ACTIONCOUNTER(i) == 5 || ACTIONCOUNTER(i) == 7) { \
							BEEP_OFF(i); \
						} else if (ACTIONCOUNTER(i) == 2 || ACTIONCOUNTER(i) == 4 || ACTIONCOUNTER(i) == 6) { \
							BEEP_ON(i); \
						} else if (ACTIONCOUNTER(i) >= 7) { \
							ACTIONTYPE(i) = 0; \
						} \
					} \

// mostly for debug
//#define PIN1_5_OFF		PRT1DR |=  BIT5
//#define PIN1_5_ON		PRT1DR &= ~BIT5

#define PIN2_6_TOGGLE 	PRT2DR ^=  BIT6
//#define PIN0_7_ON		PRT0DR &= ~BIT7
//#define PIN0_7_OFF		PRT0DR |=  BIT7

#define eprintf(format, ...) csprintf(TX_Data, format, ##__VA_ARGS__); UART_PutString(TX_Data);
#define dprintf(format) UART_CPutString(format);

void Initial(void);
void doorAction(unsigned char doorNum, unsigned char doorAction);
void setActiveReader(unsigned char doorNum);
void processBits(void);
void clearLCD(void);
void bitCleanup(void);
unsigned char strlen(char *s);


#endif  // MAIN_H_

